<?php
namespace App\Factory;

use App\Exception\ParamsException as ParamsException;

class OrderFactory {
    
    private $connection;
    private $util;
    
    public function __construct (\Doctrine\DBAL\Connection $connection,
                                 \App\Util\Util $util) {
        $this->connection = $connection;
        $this->util       = $util;
    }

    public function validateTableNumber($cdFilial, $cdLoja, $nrMesa) {
        $sql = "
            SELECT NRMESA
              FROM MESA
             WHERE CDFILIAL = :CDFILIAL
               AND CDLOJA   = :CDLOJA
               AND NRMESA   = :NRMESA
        ";

        $params = [
            ':CDFILIAL' => $cdFilial,
            ':CDLOJA'   => $cdLoja,
            ':NRMESA'   => $nrMesa
        ];

        $data = $this->connection->fetchAssoc($sql, $params);
        if(empty($data)) {
            throw new \Exception("Número de mesa informado inválido. (FILIAL: $cdFilial - LOJA: $cdLoja - NRMESA: $nrMesa)");
        }
    }

    public function orderCancel($cdFilial, $nrComanda, $nrVendaRest, $nrComandaExt) {
        $orderDetails = $this->getOrderForCancel($cdFilial, $nrComanda, $nrVendaRest, $nrComandaExt);
        
        if(empty($orderDetails))
           return 0;
        
        $this->cancelOrder($cdFilial, $nrComanda, $nrVendaRest, $nrComandaExt);
        $this->syncOrder($cdFilial, $nrVendaRest);

        return 1;
    }
    
    public function cancelOrder($cdFilial, $nrComanda, $nrVendaRest, $nrComandaExt) {
        $sql = "
            UPDATE COMANDAVEN
		       
		       SET IDSTCOMANDA = '4'
		     
		     WHERE CDFILIAL      = :CDFILIAL
	           AND NRCOMANDA     = :NRCOMANDA
	           AND NRVENDAREST   = :NRVENDAREST
	           AND NRCOMANDAEXT  = :NRCOMANDAEXT
	           AND IDORGCMDVENDA = :IDORGCMDVENDA
	    ";
        
		$params = [
			':CDFILIAL'	     => $cdFilial, 
			':NRCOMANDA'     => $nrComanda,
			':NRVENDAREST'   => $nrVendaRest,
			':NRCOMANDAEXT'  => $nrComandaExt,
			':IDORGCMDVENDA' => 'DLV_ORD'
		];
		
		$this->connection->executeQuery($sql, $params);
	}
	
	public function syncOrder($cdFilial, $nrVendaRest) {
        $sql = "
	        UPDATE VENDAREST
		       
		       SET IDSINCMSDE   = 'N'

	         WHERE CDFILIAL     = :CDFILIAL
	           AND NRVENDAREST  = :NRVENDAREST
	    ";
        
        $params = [
            ':CDFILIAL'    => $cdFilial,
            ':NRVENDAREST' => $nrVendaRest
        ];
       
        $this->connection->executeQuery($sql, $params);
	}

    public function getOrderForCancel($cdFilial, $nrComanda, $nrVendaRest, $nrComandaExt) {
        $sql = "
        	SELECT C.CDFILIAL BRANCHID, C.NRCOMANDA ORDERNUMBER, C.NRVENDAREST SELLNUMBER 
        	
        	  FROM COMANDAVEN C 

              JOIN VENDAREST VR
                ON VR.NRVENDAREST = C.NRVENDAREST
               AND VR.CDFILIAL    = C.CDFILIAL

         LEFT JOIN VENDA V
                ON C.CDFILIAL    = V.CDFILIAL
               AND C.NRVENDAREST = V.NRVENDAREST
               AND V.NRSEQVENDA IS NULL

            WHERE C.CDFILIAL     = :CDFILIAL
              AND C.NRCOMANDA    = :NRCOMANDA
              AND C.NRVENDAREST  = :NRVENDAREST
              AND C.NRCOMANDAEXT = :NRCOMANDAEXT
              AND C.IDSTCOMANDA <> '4'
              AND C.IDORGCMDVENDA IN('DLV_ORD')
              AND VR.DTHRABERMESA >= SYSDATE - 1
         ORDER BY VR.DTHRABERMESA DESC
	    ";

        $params = [
            ':CDFILIAL'     => $cdFilial,
            ':NRCOMANDA'    => $nrComanda, 
            ':NRVENDAREST'  => $nrVendaRest,
            ':NRCOMANDAEXT' => $nrComandaExt
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }

    public function getOrderStatus($cdFilial, $nrComanda, $nrVendaRest, $nrComandaExt) {
        $sql = "
            SELECT IDSTCOMANDA 
             
              FROM COMANDAVEN
             
             WHERE CDFILIAL      = :CDFILIAL
               AND NRCOMANDA     = :NRCOMANDA
               AND NRVENDAREST   = :NRVENDAREST
               AND NRCOMANDAEXT  = :NRCOMANDAEXT
               AND IDORGCMDVENDA = :IDORGCMDVENDA
        ";
        
        $params = [
            ':CDFILIAL'      => $cdFilial,
            ':NRCOMANDA'     => $nrComanda,
            ':NRVENDAREST'   => $nrVendaRest,
            ':NRCOMANDAEXT'  => $nrComandaExt,
            ':IDORGCMDVENDA' => 'DLV_ORD'
        ];

        return $this->connection->fetchAssoc($sql, $params);
    }
    
    public function insertSale($sale, $generalParams, $consumerDetails, $tableNumber) {

        $sql = "
            INSERT INTO VENDAREST
    			(CDFILIAL, NRVENDAREST, CDLOJA, NRMESAEATTAKE, CDVENDEDOR,
    			DTHRABERMESA, DTHRFECHMESA, CDOPERADOR, NRPESMESAVEN,
    			CDCLIENTE, CDCONSUMIDOR, IDPEDIDOPAGO)
    		VALUES
    			(:CDFILIAL, :NRVENDAREST, :CDLOJA, :NRMESAEATTAKE,
    			:CDVENDEDOR, TO_DATE(:CURRDATE, 'dd/mm/yyyy hh24:mi:ss'), null, :CDOPERADOR,
    			:NRPESMESAVEN, :CDCLIENTE, :CDCONSUMIDOR, 'S')";
    	
    	if(isset($generalParams["SGESTADO"]))
		    date_default_timezone_set($this->util->getCurrentTimezone($generalParams["SGESTADO"]));
		
    	$params = [
    	    ":CDFILIAL"      => $generalParams["CDFILIAL"],
            ":NRVENDAREST"   => $sale,
            ":CDLOJA"        => $generalParams["CDLOJA"],
            ":NRMESAEATTAKE" => $tableNumber,
            ":CDVENDEDOR"    => $generalParams["CDVENDEDOR"],
            ":CURRDATE"      => date("d/m/Y H:i:s"),
            ":CDOPERADOR"    => $generalParams["CDOPERADOR"],
            ":NRPESMESAVEN"  => "1",
            ":CDCLIENTE"     => $consumerDetails["CDCLIENTE"],
            ":CDCONSUMIDOR"  => $consumerDetails["CDCONSUMIDOR"]    
    	];
    	
    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertSaleCommand($orderData, $generalParams, $consumerDetails) {
        
        $sql = "
            INSERT INTO COMANDAVEN 
                (CDFILIAL, NRVENDAREST, NRCOMANDA, NRCOMANDAEXT, 
    	        CDLOJA, DSCOMANDA, IDSTCOMANDA, SGSEXOCON, 
    	        TXMOTIVCANCE, VRACRCOMANDA, CDPAIS, SGESTADO, 
    	        CDMUNICIPIO, NRCEPCONSCOMAND, CDBAIRRO, DSBAIRRO, 
    	        DSENDECONSCOMAN, NRENDECONSCOMAN, DSCOMPLENDCOCOM, DSREFENDCONSCOM, DSOBSCOMANDA,
    	        IDORGCMDVENDA, IDRETBALLOJA, NRPEDMOBILE, NRLATPEDIDO, NRLONGPEDIDO,
    	        VRDESCOMANDA, IDSUBPROJETO, DTHRAGENDADA, IDSINCAGENDA, IDIMPCUPEXP, DSCONSUMIDOR, DSEMAILENVCUPOM)
    	    VALUES 
    	        (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :NRCOMANDAEXT, 
    	        :CDLOJA, :DSCOMANDA, :IDSTCOMANDA, :SGSEXOCON, 
    	        :TXMOTIVCANCE, :VRACRCOMANDA, :CDPAIS, :SGESTADO, 
    	        :CDMUNICIPIO, :NRCEPCONSCOMAND, :CDBAIRRO, :DSBAIRRO, 
    	        :DSENDECONSCOMAN, :NRENDECONSCOMAN, :DSCOMPLENDCOCOM, :DSREFENDCONSCOM, :DSOBSCOMANDA,
    	        :IDORGCMDVENDA, :IDRETBALLOJA, :NRPEDMOBILE, :NRLATPEDIDO,
    	        :NRLONGPEDIDO, :VRDESCOMANDA, :IDSUBPROJETO, TO_DATE(:DTHRAGENDADA, 'dd/mm/yyyy hh24:mi:ss'), :IDSINCAGENDA, :IDIMPCUPEXP, :DSCONSUMIDOR, :DSEMAILENVCUPOM)";
    	
        $consumerName =  strtoupper($this->util->removeAcentos($orderData["consumerDetail"]["name"]));

        //Delivery
        $idRetBalLoja = "N";

        //Retirada balcão
        if($orderData["orderType"] == 2) {
            $idRetBalLoja = "S";
        }

        //Comer no local
        if($orderData["orderType"] == 3) {
            $idRetBalLoja = "C";
        }

    	$params = [
    	    ":CDFILIAL"        => $generalParams["CDFILIAL"],
            ":NRVENDAREST"     => $orderData["NRVENDAREST"],
            ":NRCOMANDA"       => $orderData["NRCOMANDA"],
            ":NRCOMANDAEXT"    => $orderData["orderNumber"],
            ":CDLOJA"          => $generalParams["CDLOJA"],
            ":DSCOMANDA"       => "DLV_".$orderData["NRCOMANDA"],
            ":IDSTCOMANDA"     => $orderData["IDSTCOMANDA"],
            ":SGSEXOCON"       => "M",
            ":TXMOTIVCANCE"    => "",
            ":VRACRCOMANDA"    => $this->util->toFloat($orderData["tax"]),
            ":CDPAIS"          => "0055",
            ":SGESTADO"        => strtoupper($consumerDetails["SGESTADO"]), 
            ":CDMUNICIPIO"     => $consumerDetails["CDMUNICIPIO"],
            ":NRCEPCONSCOMAND" => $consumerDetails["cep"],
            ":CDBAIRRO"        => $consumerDetails["CDBAIRRO"],
            ":DSBAIRRO"        => $consumerDetails["neighborhood"],
            ":DSENDECONSCOMAN" => substr($consumerDetails["street"], 0, 44),
            ":NRENDECONSCOMAN" => $consumerDetails["number"],
            ":DSCOMPLENDCOCOM" => $consumerDetails["complement"],
            ":DSREFENDCONSCOM" => $consumerDetails["referencePoint"],
            ":DSOBSCOMANDA"    => $orderData["notes"],
            ":IDORGCMDVENDA"   => $orderData["sourceId"],
            ":IDRETBALLOJA"    => $idRetBalLoja,
            ":NRPEDMOBILE"     => $orderData["orderNumber"],
            ":NRLATPEDIDO"     => $orderData["latitude"],
            ":NRLONGPEDIDO"    => $orderData["longitude"],
            ":VRDESCOMANDA"    => 0,
            ":IDSUBPROJETO"    => $orderData["origin"],
            ":DTHRAGENDADA"    => $orderData["DTHRAGENDADA"],
            ":IDSINCAGENDA"    => $orderData["IDSINCAGENDA"],
            ":IDIMPCUPEXP"     => $orderData["IDIMPCUPEXP"],
            ":DSCONSUMIDOR"    => substr((isset($consumerName) ? $consumerName : ""), 0, 50),
            ":DSEMAILENVCUPOM" => !empty($orderData["consumerDetail"]["email"]) ? $orderData["consumerDetail"]["email"] : null
    	];

    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertHistPedCons($orderData, $generalParams) {
        
        $sql = "
            INSERT INTO HISTPEDCONS 
            (CDFILIAL, NRVENDAREST, NRCOMANDA, NRSEQHISTPED, CDLOJA, IDHISTPEDCONS, DSHISTPEDCONS, DTHRHISTPEDCONS)
            VALUES
            (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, '001', :CDLOJA, 'R', 'Pedido Recebido', SYSDATE)";

        $params = [
    	    ":CDFILIAL"    => $generalParams["CDFILIAL"],
            ":NRVENDAREST" => $orderData["NRVENDAREST"],
            ":NRCOMANDA"   => $orderData["NRCOMANDA"],
            ":CDLOJA"      => $generalParams["CDLOJA"]
    	];

    	$this->connection->executeQuery($sql, $params);
    }
    
    public function insertCashMoviment($orderData, $payment, $generalParams, $consumerDetails) {
        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR,
        		NRCARTBANCODLV,CDNSUTEFDLV)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR,
        		:NRCARTBANCODLV, :CDNSUTEFDLV)";
        
        $params = [
            ":CDFILIAL"        => $generalParams["CDFILIAL"],
	        ":NRVENDAREST"     => $orderData["NRVENDAREST"],
	        ":NRSEQMOVDLV"     => $orderData["NRSEQMOVDLV"],
	        ":IDTIPOMOVIVEDLV" => "E",
	        ":CDCLIENTE"       => $consumerDetails["CDCLIENTE"],
	        ":CDCONSUMIDOR"    => $consumerDetails["CDCONSUMIDOR"],
	        ":CDTIPORECE"      => $orderData["CDTIPORECE"],
	        ":VRMOVIVENDDLV"   => $this->util->toFloat($payment["value"]),
	        ":NRCARTBANCODLV"  => null,
	        ":CDNSUTEFDLV"     => ( isset($payment["prepaid"]) && (!empty($payment["nsu"])) ) ? substr($payment["nsu"], 0, 50) : null
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function insertChange($orderData, $moneyReceipt, $generalParams, $consumerDetails, $movCaixa) {

        if ($orderData["troco"] < $moneyReceipt["value"])
            throw new \Exception("O valor solicitado do 'troco para' é menor do que o valor que será pago em dinheiro.");
            
        $value = $orderData["troco"] - $moneyReceipt["value"];

        $sql = "
            INSERT INTO MOVCAIXADLV
        		(CDFILIAL, NRVENDAREST, NRSEQMOVDLV, IDTIPOMOVIVEDLV,
        		VRMOVIVENDDLV, CDTIPORECE, CDCLIENTE, CDCONSUMIDOR,
        		NRCARTBANCODLV)
        	VALUES
        		(:CDFILIAL, :NRVENDAREST, :NRSEQMOVDLV, :IDTIPOMOVIVEDLV,
        		:VRMOVIVENDDLV, :CDTIPORECE, :CDCLIENTE, :CDCONSUMIDOR,
        		:NRCARTBANCODLV)";
        
        $params = [
            ":CDFILIAL"        => $generalParams["CDFILIAL"],
	        ":NRVENDAREST"     => $orderData["NRVENDAREST"],
	        ":NRSEQMOVDLV"     => $movCaixa,
	        ":IDTIPOMOVIVEDLV" => "S",
	        ":CDCLIENTE"       => $consumerDetails["CDCLIENTE"],
	        ":CDCONSUMIDOR"    => $consumerDetails["CDCONSUMIDOR"],
	        ":CDTIPORECE"      => $orderData["CDTIPORECE"],
	        ":VRMOVIVENDDLV"   => $this->util->toFloat($value),
	        ":NRCARTBANCODLV"  => null
        ];
        
        $this->connection->executeQuery($sql, $params);
    }
    
    public function getSalesWithoutOrigin() {

        $sql = "
            SELECT V.NRSEQVENDA, CV.IDSUBPROJETO, V.CDFILIAL
              FROM VENDA V
              
              JOIN VENDAREST VR
                ON V.NRVENDAREST = VR.NRVENDARESTAUX
               AND V.CDFILIAL = VR.CDFILIAL
               
              JOIN COMANDAVEN CV
                ON VR.NRVENDAREST = CV.NRVENDAREST
               AND VR.CDFILIAL = CV.CDFILIAL
               
             WHERE CV.IDORGCMDVENDA = 'DLV_ORD'
               AND V.IDSUBPROJETO IS NULL";
        
        return $this->connection->fetchAll($sql);
    }
    
    public function setSaleOrigin($saleData) {

        $sql = "
            UPDATE VENDA
               SET IDSUBPROJETO = :IDSUBPROJETO
             WHERE NRSEQVENDA   = :NRSEQVENDA
               AND CDFILIAL     = :CDFILIAL";
        
        $params = [
            ":NRSEQVENDA"   => $saleData["NRSEQVENDA"],
	        ":IDSUBPROJETO" => $saleData["IDSUBPROJETO"],
	        ":CDFILIAL"     => $saleData["CDFILIAL"]
        ];
        
        $this->connection->executeQuery($sql, $params);
    }

    //Prevents teknisa from crazy discount logics by partners ¯\_(ツ)_/¯
    public function validateOrder($cdFilial, $nrVendaRest) {
        $taxSQL = "
            SELECT VRACRCOMANDA
              FROM COMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $itemsSQL = "
            SELECT QTPRODCOMVEN, VRPRECCOMVEN, VRDESCCOMVEN
              FROM ITCOMANDAVEN
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
        ";

        $entradasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'E'
        ";

        $saidasSQL = "
            SELECT VRMOVIVENDDLV FROM MOVCAIXADLV
             WHERE NRVENDAREST      = :NRVENDAREST
               AND CDFILIAL         = :CDFILIAL
               AND IDTIPOMOVIVEDLV  = 'S'
        ";
        
        $params = [
            ":CDFILIAL"     => $cdFilial,
            ":NRVENDAREST"  => $nrVendaRest
        ];

        $taxData      = $this->connection->fetchAssoc($taxSQL, $params);
        $itemsData    = $this->connection->fetchAll($itemsSQL, $params);
        $entradasData = $this->connection->fetchAll($entradasSQL, $params);
        $saidasData   = $this->connection->fetchAll($saidasSQL, $params);

        $taxSum      = isset($taxData['VRACRCOMANDA']) ? (string)$this->formatNumber($taxData['VRACRCOMANDA']) : "0.00";
        $itemsSum    = "0.00";
        $entradasSum = "0.00";
        $saidasSum   = "0.00";

        foreach($itemsData as $itemData) {
            $mul      = bcmul($this->formatNumber($itemData['VRPRECCOMVEN']), $this->formatNumber($itemData['QTPRODCOMVEN']), 3);
            $sub      = bcsub($mul, $this->formatNumber($itemData['VRDESCCOMVEN']), 3);
            $itemsSum = bcadd($itemsSum, $sub, 3);
        }
        $itemsSum = bcadd($itemsSum, '0.00', 2);

        foreach($entradasData as $entradaData) {
            $entradasSum = bcadd($entradasSum, $this->formatNumber($entradaData['VRMOVIVENDDLV']), 2);
        }

        foreach($saidasData as $saidaData) {
            $saidasSum = bcadd($saidasSum, $this->formatNumber($saidaData['VRMOVIVENDDLV']), 2);
        }

        $orderSum    = bcadd($itemsSum, $taxSum, 2);
        $paymentsSum = bcsub($entradasSum, $saidasSum, 2);
        if(bccomp($orderSum, $paymentsSum, 2) != 0) {
            throw new ParamsException("Não foi possível aceitar o pedido. Motivo: valor do(s) item(s) enviado(s) é diferente do valor pago. Valor dos itens: {$orderSum} | Valor pago: {$paymentsSum}.");
        }
    }

    private function formatNumber($number) {
        return str_replace(",", ".", $number);
    }
    
    // Valida se o consumidor possui saldo de credito para as familias de produto
    public function getPersonalCreditValue ($generalParams) {
        $sqlCDFAMILIASALD = isset($generalParams['CDFAMILISALD']) ? 'AND (CDFAMILISALD = :CDFAMILISALD' : '';
        
        $sql = "SELECT SC.VRSALDCONFAM, SC.CDFAMILISALD, FS.NMFAMILISALD 
                    FROM SALDOCONS SC
                        JOIN FAMILIASALDO FS ON FS.CDFAMILISALD = SC.CDFAMILISALD AND SC.NRORG = FS.NRORG
                            WHERE (SC.CDCLIENTE = :CDCLIENTE)
                            AND (SC.CDCONSUMIDOR = :CDCONSUMIDOR)";
                            
        $params = [
            ":CDCLIENTE"   => $generalParams["CDCLIENTE"],
	        ":CDCONSUMIDOR" => $generalParams["CDCONSUMIDOR"],
        ];
        
        
        $resultsql = $this->connection->fetchAll($sql, $params);
        if (empty($resultsql[0])) {
            return null;
        }
        
        foreach($resultsql as &$sqlline) {
            $result = $sqlline['VRSALDCONFAM'];
            if (gettype($result) == 'string') {
                if (isset(explode(',',$result)[1]))
                    $sqlline['VRSALDCONFAM'] = floatval(str_replace(',','.',str_replace('.','',$result)));
                 else
                    $sqlline['VRSALDCONFAM'] = floatval($result);
            }
        }
        return $resultsql;
        // return count($resultsql) > 1 ? $resultsql : $resultsql[0];
    }
    
    //Busca comandas em aberto nas últimas 6 horas para o consumidor e valida se elas são de credito pessoal 
    public function getPendingSaleConsumer($generalParams, $orderData) {

         $sql = "
                SELECT CV.CDFILIAL, CV.IDSTCOMANDA, CV.NRCOMANDA,
                       CV.NRVENDAREST, VR.CDCLIENTE, VR.CDCONSUMIDOR, 
                       TO_CHAR(VR.DTHRABERMESA,'dd/mm/yyyy hh24:mi:ss') AS DTHRABERMESA
                    FROM COMANDAVEN CV , VENDAREST VR
                        WHERE CV.IDSTCOMANDA <> 'X'
                            AND CV.IDSTCOMANDA <> '4'
                            AND VR.CDCLIENTE    = :CDCLIENTE
                            AND VR.CDCONSUMIDOR = :CDCONSUMIDOR
                            AND CV.CDFILIAL     = VR.CDFILIAL
                            AND CV.NRVENDAREST  = VR.NRVENDAREST
                            AND (DTHRABERMESA BETWEEN TO_DATE(:P_INITIAL,'dd/mm/yyyy hh24:mi:ss') AND TO_DATE(:P_FINAL,'dd/mm/yyyy hh24:mi:ss'))
                            ";
                            
        $today = date("d/m/Y H:i:s");
        $limitHour = date("d/m/Y H:i:s",strtotime('-6 hour'));

        $params = [
            
            ":CDCLIENTE"    => $generalParams["CDCLIENTE"],
            ":CDCONSUMIDOR" => $generalParams["CDCONSUMIDOR"],
            ":P_INITIAL"    => $limitHour,
            ":P_FINAL"      => $today
            
            ];
            
        $resultSQL = $this->connection->fetchAll($sql, $params);
        if (empty($resultSQL))
            return null;
            
        foreach ($resultSQL as $pendingSale){
           $pendingSaleCredit = $this->ValidatePendingSale($pendingSale);
           
           if (!empty($pendingSaleCredit)){
               return $pendingSaleCredit;
               
           }
        }
        return null;
    }
    
    // Valida se existe recebimento de credito pessoal na comanda
    function ValidatePendingSale($pendingSale){
        
        $sql = "
                SELECT CDFILIAL, CDCLIENTE, CDCONSUMIDOR, 
                       CDTIPORECE, NRVENDAREST, VRMOVIVENDDLV 
                  FROM MOVCAIXADLV
                    WHERE CDFILIAL   = :CDFILIAL
                    AND NRVENDAREST  = :NRVENDAREST
                    AND CDCLIENTE    = :CDCLIENTE
                    AND CDCONSUMIDOR = :CDCONSUMIDOR
                    AND CDTIPORECE   = (SELECT MIN(CDTIPORECE) AS CDTIPORECE
                                            FROM TIPORECE
                                            WHERE IDTIPORECE = '9'
                                            AND NMTIPORECE LIKE '%CREDITO PESSOAL%')";
                                            
                                            
        $params = [
            ":CDFILIAL"     => $pendingSale["CDFILIAL"], 
            ":NRVENDAREST"  => $pendingSale["NRVENDAREST"],
            ":CDCLIENTE"    => $pendingSale["CDCLIENTE"],
            ":CDCONSUMIDOR" => $pendingSale["CDCONSUMIDOR"]
            
            ];
        $resultSQL = $this->connection->fetchAll($sql, $params);
        if (count($resultSQL) == 0){
            return null;
        }
        return $resultSQL;
    }
    
    public function getCdSubProj($origin){
        $sql = "
            SELECT CDSUBPROJETO
              FROM CADASTROSUBPROJETO
             WHERE IDSUBPROJETO = :IDSUBPROJETO
        ";
        
        $params = [
            ":IDSUBPROJETO" => $origin
        ];
        
        $url = $this->connection->fetchAssoc($sql, $params);
        
        return $url['CDSUBPROJETO'];
    }
    
    public function getSubProjectUrlOrders($origin){
        $sql = "
            SELECT WEBHOOKURL
              FROM CADASTROSUBPROJETO
             WHERE IDSUBPROJETO = :IDSUBPROJETO
        ";
        
        $params = [
            ":IDSUBPROJETO" => $origin
        ];
        
        $url = $this->connection->fetchAssoc($sql, $params);
        
        return $url['WEBHOOKURL'];
    }
    
    public function getSubProjectUrlSales($origin){
        $sql = "
            SELECT WEBHOOKURLAUX
              FROM CADASTROSUBPROJETO
             WHERE IDSUBPROJETO = :IDSUBPROJETO
        ";
        
        $params = [
            ":IDSUBPROJETO" => $origin
        ];
        
        $url = $this->connection->fetchAssoc($sql, $params);
        
        return $url['WEBHOOKURLAUX'];
    }
    
    public function getOrderHistory($subProjeto){
        if(empty($subProjeto)){
            throw new \Exception("Sub Projeto nao encontrado.");
        } else{
            $sql = "
                SELECT H.CDFILIAL, H.NRVENDAREST, H.NRCOMANDA, H.CDLOJA, CV.NRCOMANDAEXT, H.NRSEQHISTPED, H.IDHISTPEDCONS, H.DTHRHISTPEDCONS, C.IDSTATUSENVIO, CV.IDRETBALLOJA, C.CDSUBPROJETO
                  FROM HISTPEDCONS H
                  JOIN COMANDAVEN CV
                    ON CV.CDFILIAL = H.CDFILIAL
                   AND CV.NRVENDAREST = H.NRVENDAREST
                   AND CV.NRCOMANDA = H.NRCOMANDA
                   AND CV.CDLOJA = H.CDLOJA
                  LEFT JOIN (SELECT CP.*
                               FROM CONTROLEPEDIDOS CP
                               JOIN CADASTROSUBPROJETO SP
                                 ON CP.CDSUBPROJETO = SP.CDSUBPROJETO
                              WHERE SP.CDPROJETO = '007'
                                AND SP.CDSUBPROJETO = :CDSUBPROJETO) C
                    ON C.CDFILIAL = H.CDFILIAL
                   AND C.NRVENDAREST = H.NRVENDAREST
                   AND C.NRCOMANDA = H.NRCOMANDA
                   AND C.CDLOJA = H.CDLOJA
                   AND C.NRSEQHISTPED = H.NRSEQHISTPED
                 WHERE ((C.IDSTATUSENVIO = 'N') OR (C.IDSTATUSENVIO IS NULL) OR (C.IDSTATUSENVIO = 'P'))
                   AND H.IDHISTPEDCONS <> 'R'
                   AND CV.NRCOMANDAEXT IS NOT NULL
                   AND H.DTHRHISTPEDCONS > (SYSDATE - 15)
                 ORDER BY H.CDFILIAL, H.NRVENDAREST, H.NRCOMANDA, CV.NRCOMANDAEXT, H.NRSEQHISTPED
            ";
            
            $params = [
                ":CDSUBPROJETO" => $subProjeto
            ];
            
            $result = $this->connection->fetchAll($sql, $params);
        }
        
        return $result;
    }
    
    public function getSalesWithNfceAccess($subProjeto){
        if(empty($subProjeto)){
            throw new \Exception("Sub Projeto nao encontrado.");
        } else{
            $sql = "
                SELECT CV.NRCOMANDA, CV.NRCOMANDAEXT, CV.NRVENDAREST, CV.IDRETBALLOJA, C.IDSTATUSENVIO, C.CDSUBPROJETO, V.CDFILIAL, V.NRSEQVENDA, V.CDLOJA, V.CDCAIXA, V.NRACESSONFCE, NVL(V.DTENVIONFCE, V.DTVENDA) AS DTENVIONFCE
                  FROM VENDA V
                  JOIN VENDAREST VR
                    ON VR.CDFILIAL = V.CDFILIAL
                   AND VR.NRVENDARESTAUX = V.NRVENDAREST
                   AND VR.DTHRABERMESA = V.DTABERVENDA
                  JOIN COMANDAVEN CV
                    ON CV.NRVENDAREST = VR.NRVENDAREST
                   AND CV.CDFILIAL = V.CDFILIAL
                  JOIN (SELECT CVD.*
                          FROM CONTROLEVENDAS CVD
                          JOIN CADASTROSUBPROJETO SP
                            ON CVD.CDSUBPROJETO = SP.CDSUBPROJETO
                         WHERE SP.CDPROJETO = '007'
                           AND SP.CDSUBPROJETO = :CDSUBPROJETO) C
                    ON C.CDFILIAL = V.CDFILIAL
                   AND C.NRSEQVENDA = V.NRSEQVENDA
                   AND C.CDCAIXA = V.CDCAIXA
                   AND C.CDLOJA = V.CDLOJA
                 WHERE ((C.IDSTATUSENVIO = 'N') OR (C.IDSTATUSENVIO IS NULL))
                   AND V.NRACESSONFCE IS NOT NULL
                   AND V.DTVENDA > (SYSDATE - 15)
                 ORDER BY V.CDFILIAL, V.CDLOJA, V.CDCAIXA, V.DTENTRVENDA
            ";
            
            $params = [
                ":CDSUBPROJETO" => $subProjeto
            ];
            
            $result = $this->connection->fetchAll($sql, $params);
        }
        
        return $result;
    }
    
    public function insertOrderHistControl($order, $IDSTATUSENVIO, $response, $subProjeto){
        $sql = "
            SELECT * FROM CONTROLEPEDIDOS
             WHERE CDFILIAL = :CDFILIAL
               AND NRVENDAREST = :NRVENDAREST
               AND NRCOMANDA = :NRCOMANDA
               AND CDLOJA = :CDLOJA
               AND NRSEQHISTPED = :NRSEQHISTPED
               AND CDSUBPROJETO = :CDSUBPROJETO
        ";
        
        $params = [
            ":CDFILIAL" => $order['CDFILIAL'],
            ":NRVENDAREST" => $order['NRVENDAREST'],
            ":NRCOMANDA" => $order['NRCOMANDA'],
            ":CDLOJA" => $order['CDLOJA'],
            ":NRSEQHISTPED" => $order['NRSEQHISTPED'],
            ":CDSUBPROJETO" => $subProjeto
        ];
        
        $orderControl = $this->connection->fetchAssoc($sql, $params);
        
        if(empty($orderControl)){
            $sql = "
                INSERT INTO CONTROLEPEDIDOS
                (CDFILIAL, NRVENDAREST, NRCOMANDA, CDLOJA, NRSEQHISTPED, CDSUBPROJETO, IDSTATUSENVIO, DSSTATUSENVIO)
                VALUES
                (:CDFILIAL, :NRVENDAREST, :NRCOMANDA, :CDLOJA, :NRSEQHISTPED, :CDSUBPROJETO, :IDSTATUSENVIO, :DSSTATUSENVIO)
            ";
            
            $params = [
                ":CDFILIAL" => $order['CDFILIAL'],
                ":NRVENDAREST" => $order['NRVENDAREST'],
                ":NRCOMANDA" => $order['NRCOMANDA'],
                ":CDLOJA" => $order['CDLOJA'],
                ":NRSEQHISTPED" => $order['NRSEQHISTPED'],
                ":CDSUBPROJETO" => $subProjeto,
                ":IDSTATUSENVIO" => $IDSTATUSENVIO,
                ":DSSTATUSENVIO"=> $response
            ];
                
            $this->connection->executeQuery($sql, $params);
        } else{
            if(!empty($response)){
                $sql = "
                    UPDATE CONTROLEPEDIDOS
                    SET 
                        IDSTATUSENVIO = :IDSTATUSENVIO,
                        DSSTATUSENVIO = :DSSTATUSENVIO
                    WHERE CDFILIAL = :CDFILIAL
                      AND NRVENDAREST = :NRVENDAREST
                      AND NRCOMANDA = :NRCOMANDA
                      AND CDLOJA = :CDLOJA
                      AND NRSEQHISTPED = :NRSEQHISTPED
                      AND CDSUBPROJETO = :CDSUBPROJETO
                ";
                
                $params = [
                    ":CDFILIAL" => $order['CDFILIAL'],
                    ":NRVENDAREST" => $order['NRVENDAREST'],
                    ":NRCOMANDA" => $order['NRCOMANDA'],
                    ":CDLOJA" => $order['CDLOJA'],
                    ":NRSEQHISTPED" => $order['NRSEQHISTPED'],
                    ":CDSUBPROJETO" => $subProjeto,
                    ":IDSTATUSENVIO" => $IDSTATUSENVIO,
                    ":DSSTATUSENVIO"=> $response
                ];
            } else{
                $sql = "
                    UPDATE CONTROLEPEDIDOS
                    SET 
                        IDSTATUSENVIO = :IDSTATUSENVIO
                    WHERE CDFILIAL = :CDFILIAL
                      AND NRVENDAREST = :NRVENDAREST
                      AND NRCOMANDA = :NRCOMANDA
                      AND CDLOJA = :CDLOJA
                      AND NRSEQHISTPED = :NRSEQHISTPED
                      AND CDSUBPROJETO = :CDSUBPROJETO
                ";
                
                $params = [
                    ":CDFILIAL" => $order['CDFILIAL'],
                    ":NRVENDAREST" => $order['NRVENDAREST'],
                    ":NRCOMANDA" => $order['NRCOMANDA'],
                    ":CDLOJA" => $order['CDLOJA'],
                    ":NRSEQHISTPED" => $order['NRSEQHISTPED'],
                    ":CDSUBPROJETO" => $subProjeto,
                    ":IDSTATUSENVIO" => $IDSTATUSENVIO
                ];
            }
                
            $this->connection->executeQuery($sql, $params);
        }
    }
    
    public function insertSaleAcessNfceControl($sale, $IDSTATUSENVIO, $response, $subProjeto){
        $sql = "
            SELECT * FROM CONTROLEVENDAS
             WHERE CDFILIAL = :CDFILIAL
               AND NRSEQVENDA = :NRSEQVENDA
               AND CDCAIXA = :CDCAIXA
               AND CDLOJA = :CDLOJA
               AND CDSUBPROJETO = :CDSUBPROJETO
        ";
        
        $params = [
            ":CDFILIAL" => $sale['CDFILIAL'],
            ":NRSEQVENDA" => $sale['NRSEQVENDA'],
            ":CDCAIXA" => $sale['CDCAIXA'],
            ":CDLOJA" => $sale['CDLOJA'],
            ":CDSUBPROJETO" => $subProjeto
        ];
        
        $orderControl = $this->connection->fetchAssoc($sql, $params);
        
        if(empty($orderControl)){
            $sql = "
                INSERT INTO CONTROLEVENDAS
                (CDFILIAL, NRSEQVENDA, CDCAIXA, CDLOJA, CDSUBPROJETO, IDSTATUSENVIO, DSSTATUSENVIO)
                VALUES
                (:CDFILIAL, :NRSEQVENDA, :CDCAIXA, :CDLOJA, :CDSUBPROJETO, :IDSTATUSENVIO, :DSSTATUSENVIO)
            ";
            
            $params = [
                ":CDFILIAL" => $sale['CDFILIAL'],
                ":NRSEQVENDA" => $sale['NRSEQVENDA'],
                ":CDCAIXA" => $sale['CDCAIXA'],
                ":CDLOJA" => $sale['CDLOJA'],
                ":CDSUBPROJETO" => $subProjeto,
                ":IDSTATUSENVIO" => $IDSTATUSENVIO,
                ":DSSTATUSENVIO"=> $response
            ];
                
            $this->connection->executeQuery($sql, $params);
        } else{
            $sql = "
                UPDATE CONTROLEVENDAS
                SET 
                    IDSTATUSENVIO = :IDSTATUSENVIO,
                    DSSTATUSENVIO = :DSSTATUSENVIO
                WHERE CDFILIAL = :CDFILIAL
                  AND NRSEQVENDA = :NRSEQVENDA
                  AND CDCAIXA = :CDCAIXA
                  AND CDLOJA = :CDLOJA
                  AND CDSUBPROJETO = :CDSUBPROJETO
            ";
            
            $params = [
                ":CDFILIAL" => $sale['CDFILIAL'],
                ":NRSEQVENDA" => $sale['NRSEQVENDA'],
                ":CDCAIXA" => $sale['CDCAIXA'],
                ":CDLOJA" => $sale['CDLOJA'],
                ":CDSUBPROJETO" => $subProjeto,
                ":IDSTATUSENVIO" => $IDSTATUSENVIO,
                ":DSSTATUSENVIO"=> $response
            ];
                
            $this->connection->executeQuery($sql, $params);
        }
    }
    public function checkDuplicatedOrder($cdFilial, $nrPedMobile) {
        $sql = "
            SELECT CV.CDFILIAL, CV.CDLOJA, CV.NRPEDMOBILE, CV.NRCOMANDAEXT,
                   CV.IDORGCMDVENDA, TO_CHAR(VR.DTHRABERMESA, 'DD/MM/YYYY HH:MM:SS') AS DTHRABERMESA
                FROM COMANDAVEN CV
                JOIN VENDAREST VR
                    ON VR.NRVENDAREST = CV.NRVENDAREST
                WHERE CV.CDFILIAL = :CDFILIAL
                 AND CV.IDORGCMDVENDA = 'DLV_ORD'
                 AND CV.NRPEDMOBILE = :NRPEDMOBILE
                 AND (DTHRABERMESA BETWEEN TO_DATE(:P_INITIAL,'dd/mm/yyyy hh24:mi:ss') AND TO_DATE(:P_FINAL,'dd/mm/yyyy hh24:mi:ss'))
        ";
        $today = date("d/m/Y H:i:s");
        $limitHour = date("d/m/Y H:i:s",strtotime('-8 hour'));

        $params = [
            
            ":CDFILIAL"    => $cdFilial,
            ":NRPEDMOBILE" => $nrPedMobile,
            ":P_INITIAL"   => $limitHour,
            ":P_FINAL"     => $today
        ];
            
        $result = $this->connection->fetchAll($sql, $params);
        $return = false;
        
        if(count($result) > 0)
            $return = true;
            
        return $return;
    }
    
    public function getOrderSaleData($order){
        $sql = "
            SELECT V.CDFILIAL, V.CDCAIXA, V.CDLOJA, V.NRSEQVENDA
                  FROM VENDA V
                  JOIN VENDAREST VR
                    ON VR.CDFILIAL = V.CDFILIAL
                   AND VR.NRVENDARESTAUX = V.NRVENDAREST
                   AND VR.DTHRABERMESA = V.DTABERVENDA
                  JOIN COMANDAVEN CV
                    ON CV.NRVENDAREST = VR.NRVENDAREST
                   AND CV.CDFILIAL = V.CDFILIAL
                 WHERE VR.NRVENDAREST = :NRVENDAREST
                   AND VR.CDFILIAL = :CDFILIAL
                   AND CV.NRCOMANDA = :NRCOMANDA
        ";
        
        $params = [
            ":NRVENDAREST" => $order['NRVENDAREST'],
            ":CDFILIAL" => $order['CDFILIAL'],
            ":NRCOMANDA" => $order['NRCOMANDA']
        ];
        
        return $this->connection->fetchAssoc($sql, $params);
    }
}
    
   


